@@ -20,23 +20,27 @@
 #include "seccomp.h"
 #include "seize.h"
 #include "stats.h"
+#include "string.h"
 #include "xmalloc.h"
 #include "util.h"
-#include <compel/compel.h>
 
 #define NR_ATTEMPTS 5
 
-static const char frozen[]	= "FROZEN";
-static const char freezing[]	= "FREEZING";
-static const char thawed[]	= "THAWED";
+static const char frozen[] = "FROZEN";
+static const char freezing[] = "FREEZING";
+static const char thawed[] = "THAWED";
 
-static const char *get_freezer_state(int fd)
+enum freezer_state { FREEZER_ERROR = -1, THAWED, FROZEN, FREEZING };
+
+/* Track if we are running on cgroup v2 system. */
+static bool cgroup_v2 = false;
+
+static enum freezer_state get_freezer_v1_state(int fd)
 {
 	char state[32];
 	int ret;
 
-	BUILD_BUG_ON((sizeof(state) < sizeof(frozen))	||
-		     (sizeof(state) < sizeof(freezing))	||
+	BUILD_BUG_ON((sizeof(state) < sizeof(frozen)) || (sizeof(state) < sizeof(freezing)) ||
 		     (sizeof(state) < sizeof(thawed)));
 
 	lseek(fd, 0, SEEK_SET);
@@ -52,15 +56,79 @@ static const char *get_freezer_state(int fd)
 
 	pr_debug("freezer.state=%s\n", state);
 	if (strcmp(state, frozen) == 0)
-		return frozen;
+		return FROZEN;
 	else if (strcmp(state, freezing) == 0)
-		return freezing;
+		return FREEZING;
 	else if (strcmp(state, thawed) == 0)
-		return thawed;
+		return THAWED;
 
 	pr_err("Unknown freezer state: %s\n", state);
 err:
-	return NULL;
+	return FREEZER_ERROR;
+}
+
+static enum freezer_state get_freezer_v2_state(int fd)
+{
+	int exit_code = FREEZER_ERROR;
+	char path[PATH_MAX];
+	FILE *event;
+	char state;
+	int ret;
+
+	/*
+	 * cgroupv2 freezer uses cgroup.freeze to control the state. The file
+	 * can return 0 or 1. 1 means the cgroup is frozen; 0 means it is not
+	 * frozen. Writing 1 to an unfrozen cgroup can freeze it. Freezing can
+	 * take some time and if the cgroup has finished freezing can be
+	 * seen in cgroup.events: frozen 0|1.
+	 */
+
+	ret = lseek(fd, 0, SEEK_SET);
+	if (ret < 0) {
+		pr_perror("Unable to seek freezer FD");
+		goto out;
+	}
+	ret = read(fd, &state, 1);
+	if (ret <= 0) {
+		pr_perror("Unable to read from freezer FD");
+		goto out;
+	}
+	pr_debug("cgroup.freeze=%c\n", state);
+	if (state == '0') {
+		exit_code = THAWED;
+		goto out;
+	}
+
+	snprintf(path, sizeof(path), "%s/cgroup.events", opts.freeze_cgroup);
+	event = fopen(path, "r");
+	if (event == NULL) {
+		pr_perror("Unable to open %s", path);
+		goto out;
+	}
+	while (fgets(path, sizeof(path), event)) {
+		if (strncmp(path, "frozen", 6) != 0) {
+			continue;
+		} else if (strncmp(path, "frozen 0", 8) == 0) {
+			exit_code = FREEZING;
+			goto close;
+		} else if (strncmp(path, "frozen 1", 8) == 0) {
+			exit_code = FROZEN;
+			goto close;
+		}
+	}
+
+	pr_err("Unknown freezer state: %c\n", state);
+close:
+	fclose(event);
+out:
+	return exit_code;
+}
+
+static enum freezer_state get_freezer_state(int fd)
+{
+	if (cgroup_v2)
+		return get_freezer_v2_state(fd);
+	return get_freezer_v1_state(fd);
 }
 
 static bool freezer_thawed;
@@ -70,35 +138,92 @@ const char *get_real_freezer_state(void)
 	return freezer_thawed ? thawed : frozen;
 }
 
-static int freezer_restore_state(void)
+static int freezer_write_state(int fd, enum freezer_state new_state)
 {
-	int fd;
+	char state[32] = { 0 };
+	int ret;
+
+	if (new_state == THAWED) {
+		if (cgroup_v2)
+			state[0] = '0';
+		else if (strlcpy(state, thawed, sizeof(state)) >= sizeof(state))
+			return -1;
+	} else if (new_state == FROZEN) {
+		if (cgroup_v2)
+			state[0] = '1';
+		else if (strlcpy(state, frozen, sizeof(state)) >= sizeof(state))
+			return -1;
+	} else {
+		return -1;
+	}
+
+	ret = lseek(fd, 0, SEEK_SET);
+	if (ret < 0) {
+		pr_perror("Unable to seek freezer FD");
+		return -1;
+	}
+	if (write(fd, state, sizeof(state)) != sizeof(state)) {
+		pr_perror("Unable to %s tasks", (new_state == THAWED) ? "thaw" : "freeze");
+		return -1;
+	}
+
+	return 0;
+}
+
+static int freezer_open(void)
+{
+	const char freezer_v1[] = "freezer.state";
+	const char freezer_v2[] = "cgroup.freeze";
 	char path[PATH_MAX];
+	int fd;
 
-	if (!opts.freeze_cgroup || freezer_thawed)
-		return 0;
-
-	snprintf(path, sizeof(path), "%s/freezer.state", opts.freeze_cgroup);
+	snprintf(path, sizeof(path), "%s/%s", opts.freeze_cgroup, cgroup_v2 ? freezer_v2 : freezer_v1);
 	fd = open(path, O_RDWR);
 	if (fd < 0) {
 		pr_perror("Unable to open %s", path);
 		return -1;
 	}
 
-	if (write(fd, frozen, sizeof(frozen)) != sizeof(frozen)) {
-		pr_perror("Unable to freeze tasks");
-		close(fd);
+	return fd;
+}
+
+static int freezer_restore_state(void)
+{
+	int fd;
+	int ret;
+
+	if (!opts.freeze_cgroup || freezer_thawed)
+		return 0;
+
+	fd = freezer_open();
+	if (fd < 0)
 		return -1;
-	}
+
+	ret = freezer_write_state(fd, FROZEN);
 	close(fd);
-	return 0;
+	return ret;
+}
+
+static FILE *freezer_open_thread_list(char *root_path)
+{
+	char path[PATH_MAX];
+	FILE *f;
+
+	snprintf(path, sizeof(path), "%s/%s", root_path, cgroup_v2 ? "cgroup.threads" : "tasks");
+	f = fopen(path, "r");
+	if (f == NULL) {
+		pr_perror("Unable to open %s", path);
+		return NULL;
+	}
+
+	return f;
 }
 
 /* A number of tasks in a freezer cgroup which are not going to be dumped */
 static int processes_to_wait;
 static pid_t *processes_to_wait_pids;
 
-static int seize_cgroup_tree(char *root_path, const char *state)
+static int seize_cgroup_tree(char *root_path, enum freezer_state state)
 {
 	DIR *dir;
 	struct dirent *de;
@@ -109,12 +234,10 @@ static int seize_cgroup_tree(char *root_path, const char *state)
 	 * New tasks can appear while a freezer state isn't
 	 * frozen, so we need to catch all new tasks.
 	 */
-	snprintf(path, sizeof(path), "%s/tasks", root_path);
-	f = fopen(path, "r");
-	if (f == NULL) {
-		pr_perror("Unable to open %s", path);
+	f = freezer_open_thread_list(root_path);
+	if (f == NULL)
 		return -1;
-	}
+
 	while (fgets(path, sizeof(path), f)) {
 		pid_t pid;
 		int ret;
@@ -134,7 +257,7 @@ static int seize_cgroup_tree(char *root_path, const char *state)
 		if (!compel_interrupt_task(pid)) {
 			pr_debug("SEIZE %d: success\n", pid);
 			processes_to_wait++;
-		} else if (state == frozen) {
+		} else if (state == FROZEN) {
 			char buf[] = "/proc/XXXXXXXXXX/exe";
 			struct stat st;
 
@@ -194,7 +317,7 @@ static int seize_cgroup_tree(char *root_path, const char *state)
  * A freezer cgroup can contain tasks which will not be dumped
  * and we need to wait them, because the are interrupted them by ptrace.
  */
-static int freezer_wait_processes()
+static int freezer_wait_processes(void)
 {
 	int i;
 
@@ -261,12 +384,10 @@ static int log_unfrozen_stacks(char *root)
 	char path[PATH_MAX];
 	FILE *f;
 
-	snprintf(path, sizeof(path), "%s/tasks", root);
-	f = fopen(path, "r");
-	if (f == NULL) {
-		pr_perror("Unable to open %s", path);
+	f = freezer_open_thread_list(root);
+	if (f == NULL)
 		return -1;
-	}
+
 	while (fgets(path, sizeof(path), f)) {
 		pid_t pid;
 		int ret, stack;
@@ -291,7 +412,6 @@ static int log_unfrozen_stacks(char *root)
 		stackbuf[ret] = '\0';
 
 		pr_debug("Task %d has stack:\n%s", pid, stackbuf);
-
 	}
 	fclose(f);
 
@@ -331,16 +451,15 @@ static int log_unfrozen_stacks(char *root)
 static int freeze_processes(void)
 {
 	int fd, exit_code = -1;
-	char path[PATH_MAX];
-	const char *state = thawed;
+	enum freezer_state state = THAWED;
 
 	static const unsigned long step_ms = 100;
 	unsigned long nr_attempts = (opts.timeout * 1000000) / step_ms;
 	unsigned long i = 0;
 
 	const struct timespec req = {
-		.tv_nsec	= step_ms * 1000000,
-		.tv_sec		= 0,
+		.tv_nsec = step_ms * 1000000,
+		.tv_sec = 0,
 	};
 
 	if (unlikely(!nr_attempts)) {
@@ -351,26 +470,21 @@ static int freeze_processes(void)
 		nr_attempts = (10 * 1000000) / step_ms;
 	}
 
-	pr_debug("freezing processes: %lu attempts with %lu ms steps\n",
-		 nr_attempts, step_ms);
+	pr_debug("freezing processes: %lu attempts with %lu ms steps\n", nr_attempts, step_ms);
 
-	snprintf(path, sizeof(path), "%s/freezer.state", opts.freeze_cgroup);
-	fd = open(path, O_RDWR);
-	if (fd < 0) {
-		pr_perror("Unable to open %s", path);
+	fd = freezer_open();
+	if (fd < 0)
 		return -1;
-	}
+
 	state = get_freezer_state(fd);
-	if (!state) {
+	if (state == FREEZER_ERROR) {
 		close(fd);
 		return -1;
 	}
-	if (state == thawed) {
+	if (state == THAWED) {
 		freezer_thawed = true;
 
-		lseek(fd, 0, SEEK_SET);
-		if (write(fd, frozen, sizeof(frozen)) != sizeof(frozen)) {
-			pr_perror("Unable to freeze tasks");
+		if (freezer_write_state(fd, FROZEN)) {
 			close(fd);
 			return -1;
 		}
@@ -384,12 +498,12 @@ static int freeze_processes(void)
 		 */
 		for (; i <= nr_attempts; i++) {
 			state = get_freezer_state(fd);
-			if (!state) {
+			if (state == FREEZER_ERROR) {
 				close(fd);
 				return -1;
 			}
 
-			if (state == frozen)
+			if (state == FROZEN)
 				break;
 			if (alarm_timeouted())
 				goto err;
@@ -420,13 +534,9 @@ static int freeze_processes(void)
 	}
 
 err:
-	if (exit_code == 0 || freezer_thawed) {
-		lseek(fd, 0, SEEK_SET);
-		if (write(fd, thawed, sizeof(thawed)) != sizeof(thawed)) {
-			pr_perror("Unable to thaw tasks");
-			exit_code = -1;
-		}
-	}
+	if (exit_code == 0 || freezer_thawed)
+		exit_code = freezer_write_state(fd, THAWED);
+
 	if (close(fd)) {
 		pr_perror("Unable to thaw tasks");
 		return -1;
@@ -483,7 +593,7 @@ static int collect_children(struct pstree_item *item)
 
 		if (!opts.freeze_cgroup)
 			/* fails when meets a zombie */
-			compel_interrupt_task(pid);
+			__ignore_value(compel_interrupt_task(pid));
 
 		ret = compel_wait_task(pid, item->pid->real, parse_pid_status, NULL, &creds.s, NULL);
 		if (ret < 0) {
@@ -551,7 +661,6 @@ static void pstree_wait(struct pstree_item *root_item)
 	int pid, status, i;
 
 	for_each_pstree_item(item) {
-
 		if (item->pid->state == TASK_DEAD)
 			continue;
 
@@ -562,8 +671,7 @@ static void pstree_wait(struct pstree_item *root_item)
 				break;
 			} else {
 				if (!WIFSIGNALED(status) || WTERMSIG(status) != SIGKILL) {
-					pr_err("Unexpected exit code %d of %d: %s\n",
-						status, pid, strsignal(status));
+					pr_err("Unexpected exit code %d of %d: %s\n", status, pid, strsignal(status));
 					BUG();
 				}
 			}
@@ -666,8 +774,7 @@ static int collect_threads(struct pstree_item *item)
 
 		nr_inprogress++;
 
-		pr_info("\tSeizing %d's %d thread\n",
-				item->pid->real, pid);
+		pr_info("\tSeizing %d's %d thread\n", item->pid->real, pid);
 
 		if (!opts.freeze_cgroup && compel_interrupt_task(pid))
 			continue;
@@ -691,6 +798,7 @@ static int collect_threads(struct pstree_item *item)
 
 		BUG_ON(item->nr_threads + 1 > nr_threads);
 		item->threads[item->nr_threads].real = pid;
+		item->threads[item->nr_threads].ns[0].virt = t_creds.s.vpid;
 		item->threads[item->nr_threads].item = NULL;
 		item->nr_threads++;
 
@@ -720,8 +828,7 @@ err:
 	return -1;
 }
 
-static int collect_loop(struct pstree_item *item,
-		int (*collect)(struct pstree_item *))
+static int collect_loop(struct pstree_item *item, int (*collect)(struct pstree_item *))
 {
 	int attempts = NR_ATTEMPTS, nr_inprogress = 1;
 
@@ -784,6 +891,27 @@ err_close:
 	return -1;
 }
 
+static int cgroup_version(void)
+{
+	char path[PATH_MAX];
+
+	snprintf(path, sizeof(path), "%s/freezer.state", opts.freeze_cgroup);
+	if (access(path, F_OK) == 0) {
+		cgroup_v2 = false;
+		return 0;
+	}
+
+	snprintf(path, sizeof(path), "%s/cgroup.freeze", opts.freeze_cgroup);
+	if (access(path, F_OK) == 0) {
+		cgroup_v2 = true;
+		return 0;
+	}
+
+	pr_err("Neither a cgroupv1 (freezer.state) or cgroupv2 (cgroup.freeze) control file found.\n");
+
+	return -1;
+}
+
 int collect_pstree(void)
 {
 	pid_t pid = root_item->pid->real;
@@ -799,6 +927,11 @@ int collect_pstree(void)
 	 */
 	alarm(opts.timeout);
 
+	if (opts.freeze_cgroup && cgroup_version())
+		goto err;
+
+	pr_debug("Detected cgroup V%d freezer\n", cgroup_v2 ? 2 : 1);
+
 	if (opts.freeze_cgroup && freeze_processes())
 		goto err;
 
@@ -841,4 +974,3 @@ err:
 	alarm(0);
 	return ret;
 }
-
