@@ -30,17 +30,17 @@
 #include "util.h"
 #include "fdstore.h"
 
-#undef  LOG_PREFIX
+#undef LOG_PREFIX
 #define LOG_PREFIX "sockets: "
 
 #ifndef SOCK_DIAG_BY_FAMILY
 #define SOCK_DIAG_BY_FAMILY 20
 #endif
 
-#define SK_HASH_SIZE		32
+#define SK_HASH_SIZE 32
 
 #ifndef SO_GET_FILTER
-#define SO_GET_FILTER	SO_ATTACH_FILTER
+#define SO_GET_FILTER SO_ATTACH_FILTER
 #endif
 
 static const char *__socket_const_name(char *dst, size_t len, const char **a, size_t n, unsigned int v)
@@ -57,21 +57,14 @@ static const char *__socket_const_name(char *dst, size_t len, const char **a, si
 const char *socket_proto_name(unsigned int proto, char *nm, size_t size)
 {
 	static const char *protos[] = {
-		[IPPROTO_IP]		= __stringify_1(IPPROTO_IP),
-		[IPPROTO_ICMP]		= __stringify_1(IPPROTO_ICMP),
-		[IPPROTO_IGMP]		= __stringify_1(IPPROTO_IGMP),
-		[IPPROTO_IPIP]		= __stringify_1(IPPROTO_IPIP),
-		[IPPROTO_TCP]		= __stringify_1(IPPROTO_TCP),
-		[IPPROTO_EGP]		= __stringify_1(IPPROTO_EGP),
-		[IPPROTO_UDP]		= __stringify_1(IPPROTO_UDP),
-		[IPPROTO_DCCP]		= __stringify_1(IPPROTO_DCCP),
-		[IPPROTO_IPV6]		= __stringify_1(IPPROTO_IPV6),
-		[IPPROTO_RSVP]		= __stringify_1(IPPROTO_RSVP),
-		[IPPROTO_GRE]		= __stringify_1(IPPROTO_GRE),
-		[IPPROTO_ESP]		= __stringify_1(IPPROTO_ESP),
-		[IPPROTO_AH]		= __stringify_1(IPPROTO_AH),
-		[IPPROTO_UDPLITE]	= __stringify_1(IPPROTO_UDPLITE),
-		[IPPROTO_RAW]		= __stringify_1(IPPROTO_RAW),
+		[IPPROTO_IP] = __stringify_1(IPPROTO_IP),     [IPPROTO_ICMP] = __stringify_1(IPPROTO_ICMP),
+		[IPPROTO_IGMP] = __stringify_1(IPPROTO_IGMP), [IPPROTO_IPIP] = __stringify_1(IPPROTO_IPIP),
+		[IPPROTO_TCP] = __stringify_1(IPPROTO_TCP),   [IPPROTO_EGP] = __stringify_1(IPPROTO_EGP),
+		[IPPROTO_UDP] = __stringify_1(IPPROTO_UDP),   [IPPROTO_DCCP] = __stringify_1(IPPROTO_DCCP),
+		[IPPROTO_IPV6] = __stringify_1(IPPROTO_IPV6), [IPPROTO_RSVP] = __stringify_1(IPPROTO_RSVP),
+		[IPPROTO_GRE] = __stringify_1(IPPROTO_GRE),   [IPPROTO_ESP] = __stringify_1(IPPROTO_ESP),
+		[IPPROTO_AH] = __stringify_1(IPPROTO_AH),     [IPPROTO_UDPLITE] = __stringify_1(IPPROTO_UDPLITE),
+		[IPPROTO_RAW] = __stringify_1(IPPROTO_RAW),
 	};
 	return __socket_const_name(nm, size, protos, ARRAY_SIZE(protos), proto);
 }
@@ -79,13 +72,10 @@ const char *socket_proto_name(unsigned int proto, char *nm, size_t size)
 const char *socket_family_name(unsigned int family, char *nm, size_t size)
 {
 	static const char *families[] = {
-		[AF_UNIX]		= __stringify_1(AF_UNIX),
-		[AF_INET]		= __stringify_1(AF_INET),
-		[AF_BRIDGE]		= __stringify_1(AF_BRIDGE),
-		[AF_INET6]		= __stringify_1(AF_INET6),
-		[AF_KEY]		= __stringify_1(AF_KEY),
-		[AF_NETLINK]		= __stringify_1(AF_NETLINK),
-		[AF_PACKET]		= __stringify_1(AF_PACKET),
+		[AF_UNIX] = __stringify_1(AF_UNIX),	[AF_INET] = __stringify_1(AF_INET),
+		[AF_BRIDGE] = __stringify_1(AF_BRIDGE), [AF_INET6] = __stringify_1(AF_INET6),
+		[AF_KEY] = __stringify_1(AF_KEY),	[AF_NETLINK] = __stringify_1(AF_NETLINK),
+		[AF_PACKET] = __stringify_1(AF_PACKET),
 	};
 	return __socket_const_name(nm, size, families, ARRAY_SIZE(families), family);
 }
@@ -93,11 +83,9 @@ const char *socket_family_name(unsigned int family, char *nm, size_t size)
 const char *socket_type_name(unsigned int type, char *nm, size_t size)
 {
 	static const char *types[] = {
-		[SOCK_STREAM]		= __stringify_1(SOCK_STREAM),
-		[SOCK_DGRAM]		= __stringify_1(SOCK_DGRAM),
-		[SOCK_RAW]		= __stringify_1(SOCK_RAW),
-		[SOCK_SEQPACKET]	= __stringify_1(SOCK_SEQPACKET),
-		[SOCK_PACKET]		= __stringify_1(SOCK_PACKET),
+		[SOCK_STREAM] = __stringify_1(SOCK_STREAM), [SOCK_DGRAM] = __stringify_1(SOCK_DGRAM),
+		[SOCK_RAW] = __stringify_1(SOCK_RAW),	    [SOCK_SEQPACKET] = __stringify_1(SOCK_SEQPACKET),
+		[SOCK_PACKET] = __stringify_1(SOCK_PACKET),
 	};
 	return __socket_const_name(nm, size, types, ARRAY_SIZE(types), type);
 }
@@ -105,39 +93,38 @@ const char *socket_type_name(unsigned int type, char *nm, size_t size)
 const char *tcp_state_name(unsigned int state, char *nm, size_t size)
 {
 	static const char *states[] = {
-		[TCP_ESTABLISHED]	= __stringify_1(TCP_ESTABLISHED),
-		[TCP_SYN_SENT]		= __stringify_1(TCP_SYN_SENT),
-		[TCP_SYN_RECV]		= __stringify_1(TCP_SYN_RECV),
-		[TCP_FIN_WAIT1]		= __stringify_1(TCP_FIN_WAIT1),
-		[TCP_FIN_WAIT2]		= __stringify_1(TCP_FIN_WAIT2),
-		[TCP_TIME_WAIT]		= __stringify_1(TCP_TIME_WAIT),
-		[TCP_CLOSE]		= __stringify_1(TCP_CLOSE),
-		[TCP_CLOSE_WAIT]	= __stringify_1(TCP_CLOSE_WAIT),
-		[TCP_LAST_ACK]		= __stringify_1(TCP_LAST_ACK),
-		[TCP_LISTEN]		= __stringify_1(TCP_LISTEN),
-		[TCP_CLOSING]		= __stringify_1(TCP_CLOSING),
+		[TCP_ESTABLISHED] = __stringify_1(TCP_ESTABLISHED),
+		[TCP_SYN_SENT] = __stringify_1(TCP_SYN_SENT),
+		[TCP_SYN_RECV] = __stringify_1(TCP_SYN_RECV),
+		[TCP_FIN_WAIT1] = __stringify_1(TCP_FIN_WAIT1),
+		[TCP_FIN_WAIT2] = __stringify_1(TCP_FIN_WAIT2),
+		[TCP_TIME_WAIT] = __stringify_1(TCP_TIME_WAIT),
+		[TCP_CLOSE] = __stringify_1(TCP_CLOSE),
+		[TCP_CLOSE_WAIT] = __stringify_1(TCP_CLOSE_WAIT),
+		[TCP_LAST_ACK] = __stringify_1(TCP_LAST_ACK),
+		[TCP_LISTEN] = __stringify_1(TCP_LISTEN),
+		[TCP_CLOSING] = __stringify_1(TCP_CLOSING),
 	};
 	return __socket_const_name(nm, size, states, ARRAY_SIZE(states), state);
 }
 
 struct sock_diag_greq {
-	u8	family;
-	u8	protocol;
+	u8 family;
+	u8 protocol;
 };
 
 struct sock_diag_req {
 	struct nlmsghdr hdr;
 	union {
-		struct unix_diag_req	u;
-		struct inet_diag_req_v2	i;
-		struct packet_diag_req	p;
+		struct unix_diag_req u;
+		struct inet_diag_req_v2 i;
+		struct packet_diag_req p;
 		struct netlink_diag_req n;
-		struct sock_diag_greq	g;
+		struct sock_diag_greq g;
 	} r;
 };
 
-enum socket_cl_bits
-{
+enum socket_cl_bits {
 	NETLINK_CL_BIT,
 	INET_TCP_CL_BIT,
 	INET_UDP_CL_BIT,
@@ -156,8 +143,7 @@ enum socket_cl_bits
 
 static DECLARE_BITMAP(socket_cl_bits, MAX_CL_BIT);
 
-static inline
-enum socket_cl_bits get_collect_bit_nr(unsigned int family, unsigned int proto)
+static inline enum socket_cl_bits get_collect_bit_nr(unsigned int family, unsigned int proto)
 {
 	if (family == AF_NETLINK)
 		return NETLINK_CL_BIT;
@@ -252,8 +238,8 @@ void preload_socket_modules(void)
 	pr_info("Probing sock diag modules\n");
 
 	memset(&req, 0, sizeof(req));
-	req.hdr.nlmsg_type	= SOCK_DIAG_BY_FAMILY;
-	req.hdr.nlmsg_seq	= CR_NLMSG_SEQ;
+	req.hdr.nlmsg_type = SOCK_DIAG_BY_FAMILY;
+	req.hdr.nlmsg_seq = CR_NLMSG_SEQ;
 
 	/*
 	 * Probe UNIX, netlink and packet diag-s by feeding
@@ -346,10 +332,7 @@ static void encode_filter(struct sock_filter *f, u64 *img, int n)
 	BUILD_BUG_ON(sizeof(*f) != sizeof(*img));
 
 	for (i = 0; i < n; i++)
-		img[i] = ((u64)f[i].code << 48) |
-			 ((u64)f[i].jt << 40) |
-			 ((u64)f[i].jf << 32) |
-			 ((u64)f[i].k << 0);
+		img[i] = ((u64)f[i].code << 48) | ((u64)f[i].jt << 40) | ((u64)f[i].jf << 32) | ((u64)f[i].k << 0);
 }
 
 static void decode_filter(u64 *img, struct sock_filter *f, int n)
@@ -414,12 +397,15 @@ static int restore_socket_filter(int sk, SkOptsEntry *soe)
 
 	pr_info("Restoring socket filter\n");
 	sfp.len = soe->n_so_filter;
-	sfp.filter = xmalloc(soe->n_so_filter * sfp.len);
+	sfp.filter = xmalloc(sfp.len * sizeof(struct sock_filter));
 	if (!sfp.filter)
 		return -1;
 
 	decode_filter(soe->so_filter, sfp.filter, sfp.len);
 	ret = restore_opt(sk, SOL_SOCKET, SO_ATTACH_FILTER, &sfp);
+	if (ret)
+		pr_err("Can't restore filter\n");
+
 	xfree(sfp.filter);
 
 	return ret;
@@ -443,12 +429,10 @@ struct socket_desc *lookup_socket_ino(unsigned int ino, int family)
 	return NULL;
 }
 
-
 struct socket_desc *lookup_socket(unsigned int ino, int family, int proto)
 {
 	if (!socket_test_collect_bit(family, proto)) {
-		pr_err("Sockets (family %d proto %d) are not collected\n",
-		       family, proto);
+		pr_err("Sockets (family %d proto %d) are not collected\n", family, proto);
 		return ERR_PTR(-EINVAL);
 	}
 
@@ -459,10 +443,10 @@ int sk_collect_one(unsigned ino, int family, struct socket_desc *d, struct ns_id
 {
 	struct socket_desc **chain;
 
-	d->ino		= ino;
-	d->family	= family;
+	d->ino = ino;
+	d->family = family;
 	d->already_dumped = 0;
-	d->sk_ns	= ns;
+	d->sk_ns = ns;
 
 	chain = &sockets[ino % SK_HASH_SIZE];
 	d->next = *chain;
@@ -514,7 +498,7 @@ int restore_prepare_socket(int sk)
 		pr_perror("Unable to get flags for %d", sk);
 		return -1;
 	}
-	if (fcntl(sk, F_SETFL, flags | O_NONBLOCK) ) {
+	if (fcntl(sk, F_SETFL, flags | O_NONBLOCK)) {
 		pr_perror("Unable to set O_NONBLOCK for %d", sk);
 		return -1;
 	}
@@ -524,10 +508,11 @@ int restore_prepare_socket(int sk)
 
 int restore_socket_opts(int sk, SkOptsEntry *soe)
 {
-	int ret = 0, val;
+	int ret = 0, val = 1;
 	struct timeval tv;
+	struct linger so_linger;
 	/* In kernel a bufsize value is doubled. */
-	u32 bufs[2] = { soe->so_sndbuf / 2, soe->so_rcvbuf / 2};
+	u32 bufs[2] = { soe->so_sndbuf / 2, soe->so_rcvbuf / 2 };
 
 	pr_info("%d restore sndbuf %d rcv buf %d\n", sk, soe->so_sndbuf, soe->so_rcvbuf);
 
@@ -547,30 +532,51 @@ int restore_socket_opts(int sk, SkOptsEntry *soe)
 		ret |= restore_opt(sk, SOL_SOCKET, SO_MARK, &soe->so_mark);
 	}
 	if (soe->has_so_passcred && soe->so_passcred) {
-		val = 1;
 		pr_debug("\tset passcred for socket\n");
 		ret |= restore_opt(sk, SOL_SOCKET, SO_PASSCRED, &val);
 	}
 	if (soe->has_so_passsec && soe->so_passsec) {
-		val = 1;
 		pr_debug("\tset passsec for socket\n");
 		ret |= restore_opt(sk, SOL_SOCKET, SO_PASSSEC, &val);
 	}
 	if (soe->has_so_dontroute && soe->so_dontroute) {
-		val = 1;
 		pr_debug("\tset dontroute for socket\n");
 		ret |= restore_opt(sk, SOL_SOCKET, SO_DONTROUTE, &val);
 	}
 	if (soe->has_so_no_check && soe->so_no_check) {
-		val = 1;
 		pr_debug("\tset no_check for socket\n");
 		ret |= restore_opt(sk, SOL_SOCKET, SO_NO_CHECK, &val);
 	}
 	if (soe->has_so_broadcast && soe->so_broadcast) {
-		val = 1;
 		pr_debug("\tset broadcast for socket\n");
 		ret |= restore_opt(sk, SOL_SOCKET, SO_BROADCAST, &val);
 	}
+	if (soe->has_so_oobinline && soe->so_oobinline) {
+		pr_debug("\tset oobinline for socket\n");
+		ret |= restore_opt(sk, SOL_SOCKET, SO_OOBINLINE, &val);
+	}
+	if (soe->has_so_linger) {
+		pr_debug("\tset so_linger for socket\n");
+		so_linger.l_onoff = true;
+		so_linger.l_linger = soe->so_linger;
+		ret |= restore_opt(sk, SOL_SOCKET, SO_LINGER, &so_linger);
+	}
+	if (soe->has_so_keepalive && soe->so_keepalive) {
+		pr_debug("\tset keepalive for socket\n");
+		ret |= restore_opt(sk, SOL_SOCKET, SO_KEEPALIVE, &val);
+	}
+	if (soe->has_tcp_keepcnt) {
+		pr_debug("\tset keepcnt for socket\n");
+		ret |= restore_opt(sk, SOL_TCP, TCP_KEEPCNT, &soe->tcp_keepcnt);
+	}
+	if (soe->has_tcp_keepidle) {
+		pr_debug("\tset keepidle for socket\n");
+		ret |= restore_opt(sk, SOL_TCP, TCP_KEEPIDLE, &soe->tcp_keepidle);
+	}
+	if (soe->has_tcp_keepintvl) {
+		pr_debug("\tset keepintvl for socket\n");
+		ret |= restore_opt(sk, SOL_TCP, TCP_KEEPINTVL, &soe->tcp_keepintvl);
+	}
 
 	tv.tv_sec = soe->so_snd_tmo_sec;
 	tv.tv_usec = soe->so_snd_tmo_usec;
@@ -598,8 +604,7 @@ int do_dump_opt(int sk, int level, int name, void *val, int len)
 	}
 
 	if (aux != len) {
-		pr_err("Len mismatch on %d:%d : %d, want %d\n",
-				level, name, aux, len);
+		pr_err("Len mismatch on %d:%d : %d, want %d\n", level, name, aux, len);
 		return -1;
 	}
 
@@ -610,6 +615,7 @@ int dump_socket_opts(int sk, SkOptsEntry *soe)
 {
 	int ret = 0, val;
 	struct timeval tv;
+	struct linger so_linger = { 0, 0 };
 
 	ret |= dump_opt(sk, SOL_SOCKET, SO_SNDBUF, &soe->so_sndbuf);
 	ret |= dump_opt(sk, SOL_SOCKET, SO_RCVBUF, &soe->so_rcvbuf);
@@ -656,6 +662,20 @@ int dump_socket_opts(int sk, SkOptsEntry *soe)
 	soe->has_so_broadcast = true;
 	soe->so_broadcast = val ? true : false;
 
+	ret |= dump_opt(sk, SOL_SOCKET, SO_KEEPALIVE, &val);
+	soe->has_so_keepalive = true;
+	soe->so_keepalive = val ? true : false;
+
+	ret |= dump_opt(sk, SOL_SOCKET, SO_OOBINLINE, &val);
+	soe->has_so_oobinline = true;
+	soe->so_oobinline = val ? true : false;
+
+	ret |= dump_opt(sk, SOL_SOCKET, SO_LINGER, &so_linger);
+	if (so_linger.l_onoff) {
+		soe->has_so_linger = true;
+		soe->so_linger = so_linger.l_linger;
+	}
+
 	ret |= dump_bound_dev(sk, soe);
 	ret |= dump_socket_filter(sk, soe);
 
@@ -728,9 +748,8 @@ static int inet_receive_one(struct nlmsghdr *h, struct ns_id *ns, void *arg)
 }
 
 static int do_collect_req(int nl, struct sock_diag_req *req, int size,
-		int (*receive_callback)(struct nlmsghdr *h, struct ns_id *ns, void *),
-		int (*error_callback)(int err, struct ns_id *ns, void *),
-		struct ns_id *ns, void *arg)
+			  int (*receive_callback)(struct nlmsghdr *h, struct ns_id *ns, void *),
+			  int (*error_callback)(int err, struct ns_id *ns, void *), struct ns_id *ns, void *arg)
 {
 	int tmp = do_rtnl_req(nl, req, size, receive_callback, error_callback, ns, arg);
 	if (tmp == 0)
@@ -744,8 +763,7 @@ static int collect_err(int err, struct ns_id *ns, void *arg)
 	char family[32], proto[32];
 	char msg[256];
 
-	snprintf(msg, sizeof(msg),
-		 "Sockects collect procedure family %s proto %s",
+	snprintf(msg, sizeof(msg), "Sockects collect procedure family %s proto %s",
 		 socket_family_name(gr->family, family, sizeof(family)),
 		 socket_proto_name(gr->protocol, proto, sizeof(proto)));
 
@@ -776,9 +794,7 @@ static int collect_err(int err, struct ns_id *ns, void *arg)
 	 * may return EINVAL on older kernels.
 	 */
 	if (err == -EINVAL) {
-		if (gr->family == AF_UNIX ||
-		    gr->family == AF_PACKET ||
-		    gr->family == AF_NETLINK) {
+		if (gr->family == AF_UNIX || gr->family == AF_PACKET || gr->family == AF_NETLINK) {
 			pr_debug("%s: %d\n", msg, err);
 			return -EINVAL;
 		}
@@ -803,112 +819,107 @@ int collect_sockets(struct ns_id *ns)
 	struct sock_diag_req req;
 
 	memset(&req, 0, sizeof(req));
-	req.hdr.nlmsg_len	= sizeof(req);
-	req.hdr.nlmsg_type	= SOCK_DIAG_BY_FAMILY;
-	req.hdr.nlmsg_flags	= NLM_F_DUMP | NLM_F_REQUEST;
-	req.hdr.nlmsg_seq	= CR_NLMSG_SEQ;
+	req.hdr.nlmsg_len = sizeof(req);
+	req.hdr.nlmsg_type = SOCK_DIAG_BY_FAMILY;
+	req.hdr.nlmsg_flags = NLM_F_DUMP | NLM_F_REQUEST;
+	req.hdr.nlmsg_seq = CR_NLMSG_SEQ;
 
 	/* Collect UNIX sockets */
-	req.r.u.sdiag_family	= AF_UNIX;
-	req.r.u.udiag_states	= -1; /* All */
-	req.r.u.udiag_show	= UDIAG_SHOW_NAME | UDIAG_SHOW_VFS |
-				  UDIAG_SHOW_PEER | UDIAG_SHOW_ICONS |
-				  UDIAG_SHOW_RQLEN;
+	req.r.u.sdiag_family = AF_UNIX;
+	req.r.u.udiag_states = -1; /* All */
+	req.r.u.udiag_show = UDIAG_SHOW_NAME | UDIAG_SHOW_VFS | UDIAG_SHOW_PEER | UDIAG_SHOW_ICONS | UDIAG_SHOW_RQLEN;
 	tmp = do_collect_req(nl, &req, sizeof(req), unix_receive_one, collect_err, ns, &req.r.u);
 	if (tmp)
 		err = tmp;
 
 	/* Collect IPv4 TCP sockets */
-	req.r.i.sdiag_family	= AF_INET;
-	req.r.i.sdiag_protocol	= IPPROTO_TCP;
-	req.r.i.idiag_ext	= 0;
+	req.r.i.sdiag_family = AF_INET;
+	req.r.i.sdiag_protocol = IPPROTO_TCP;
+	req.r.i.idiag_ext = 0;
 	/* Only listening and established sockets supported yet */
-	req.r.i.idiag_states	= (1 << TCP_LISTEN) | (1 << TCP_ESTABLISHED) |
-					(1 << TCP_FIN_WAIT1) | (1 << TCP_FIN_WAIT2) |
-					(1 << TCP_CLOSE_WAIT) | (1 << TCP_LAST_ACK) |
-					(1 << TCP_CLOSING) | (1 << TCP_SYN_SENT);
+	req.r.i.idiag_states = (1 << TCP_LISTEN) | (1 << TCP_ESTABLISHED) | (1 << TCP_FIN_WAIT1) |
+			       (1 << TCP_FIN_WAIT2) | (1 << TCP_CLOSE_WAIT) | (1 << TCP_LAST_ACK) | (1 << TCP_CLOSING) |
+			       (1 << TCP_SYN_SENT);
 	tmp = do_collect_req(nl, &req, sizeof(req), inet_receive_one, collect_err, ns, &req.r.i);
 	if (tmp)
 		err = tmp;
 
 	/* Collect IPv4 UDP sockets */
-	req.r.i.sdiag_family	= AF_INET;
-	req.r.i.sdiag_protocol	= IPPROTO_UDP;
-	req.r.i.idiag_ext	= 0;
-	req.r.i.idiag_states	= -1; /* All */
+	req.r.i.sdiag_family = AF_INET;
+	req.r.i.sdiag_protocol = IPPROTO_UDP;
+	req.r.i.idiag_ext = 0;
+	req.r.i.idiag_states = -1; /* All */
 	tmp = do_collect_req(nl, &req, sizeof(req), inet_receive_one, collect_err, ns, &req.r.i);
 	if (tmp)
 		err = tmp;
 
 	/* Collect IPv4 UDP-lite sockets */
-	req.r.i.sdiag_family	= AF_INET;
-	req.r.i.sdiag_protocol	= IPPROTO_UDPLITE;
-	req.r.i.idiag_ext	= 0;
-	req.r.i.idiag_states	= -1; /* All */
+	req.r.i.sdiag_family = AF_INET;
+	req.r.i.sdiag_protocol = IPPROTO_UDPLITE;
+	req.r.i.idiag_ext = 0;
+	req.r.i.idiag_states = -1; /* All */
 	tmp = do_collect_req(nl, &req, sizeof(req), inet_receive_one, collect_err, ns, &req.r.i);
 	if (tmp)
 		err = tmp;
 
 	/* Collect IPv4 RAW sockets */
-	req.r.i.sdiag_family	= AF_INET;
-	req.r.i.sdiag_protocol	= IPPROTO_RAW;
-	req.r.i.idiag_ext	= 0;
-	req.r.i.idiag_states	= -1; /* All */
+	req.r.i.sdiag_family = AF_INET;
+	req.r.i.sdiag_protocol = IPPROTO_RAW;
+	req.r.i.idiag_ext = 0;
+	req.r.i.idiag_states = -1; /* All */
 	tmp = do_collect_req(nl, &req, sizeof(req), inet_receive_one, collect_err, ns, &req.r.i);
 	if (tmp)
 		err = tmp;
 
 	/* Collect IPv6 TCP sockets */
-	req.r.i.sdiag_family	= AF_INET6;
-	req.r.i.sdiag_protocol	= IPPROTO_TCP;
-	req.r.i.idiag_ext	= 0;
+	req.r.i.sdiag_family = AF_INET6;
+	req.r.i.sdiag_protocol = IPPROTO_TCP;
+	req.r.i.idiag_ext = 0;
 	/* Only listening sockets supported yet */
-	req.r.i.idiag_states	= (1 << TCP_LISTEN) | (1 << TCP_ESTABLISHED) |
-					(1 << TCP_FIN_WAIT1) | (1 << TCP_FIN_WAIT2) |
-					(1 << TCP_CLOSE_WAIT) | (1 << TCP_LAST_ACK) |
-					(1 << TCP_CLOSING) | (1 << TCP_SYN_SENT);
+	req.r.i.idiag_states = (1 << TCP_LISTEN) | (1 << TCP_ESTABLISHED) | (1 << TCP_FIN_WAIT1) |
+			       (1 << TCP_FIN_WAIT2) | (1 << TCP_CLOSE_WAIT) | (1 << TCP_LAST_ACK) | (1 << TCP_CLOSING) |
+			       (1 << TCP_SYN_SENT);
 	tmp = do_collect_req(nl, &req, sizeof(req), inet_receive_one, collect_err, ns, &req.r.i);
 	if (tmp)
 		err = tmp;
 
 	/* Collect IPv6 UDP sockets */
-	req.r.i.sdiag_family	= AF_INET6;
-	req.r.i.sdiag_protocol	= IPPROTO_UDP;
-	req.r.i.idiag_ext	= 0;
-	req.r.i.idiag_states	= -1; /* All */
+	req.r.i.sdiag_family = AF_INET6;
+	req.r.i.sdiag_protocol = IPPROTO_UDP;
+	req.r.i.idiag_ext = 0;
+	req.r.i.idiag_states = -1; /* All */
 	tmp = do_collect_req(nl, &req, sizeof(req), inet_receive_one, collect_err, ns, &req.r.i);
 	if (tmp)
 		err = tmp;
 
 	/* Collect IPv6 UDP-lite sockets */
-	req.r.i.sdiag_family	= AF_INET6;
-	req.r.i.sdiag_protocol	= IPPROTO_UDPLITE;
-	req.r.i.idiag_ext	= 0;
-	req.r.i.idiag_states	= -1; /* All */
+	req.r.i.sdiag_family = AF_INET6;
+	req.r.i.sdiag_protocol = IPPROTO_UDPLITE;
+	req.r.i.idiag_ext = 0;
+	req.r.i.idiag_states = -1; /* All */
 	tmp = do_collect_req(nl, &req, sizeof(req), inet_receive_one, collect_err, ns, &req.r.i);
 	if (tmp)
 		err = tmp;
 
 	/* Collect IPv6 RAW sockets */
-	req.r.i.sdiag_family	= AF_INET6;
-	req.r.i.sdiag_protocol	= IPPROTO_RAW;
-	req.r.i.idiag_ext	= 0;
-	req.r.i.idiag_states	= -1; /* All */
+	req.r.i.sdiag_family = AF_INET6;
+	req.r.i.sdiag_protocol = IPPROTO_RAW;
+	req.r.i.idiag_ext = 0;
+	req.r.i.idiag_states = -1; /* All */
 	tmp = do_collect_req(nl, &req, sizeof(req), inet_receive_one, collect_err, ns, &req.r.i);
 	if (tmp)
 		err = tmp;
 
-	req.r.p.sdiag_family	= AF_PACKET;
-	req.r.p.sdiag_protocol	= 0;
-	req.r.p.pdiag_show	= PACKET_SHOW_INFO | PACKET_SHOW_MCLIST |
-					PACKET_SHOW_FANOUT | PACKET_SHOW_RING_CFG;
+	req.r.p.sdiag_family = AF_PACKET;
+	req.r.p.sdiag_protocol = 0;
+	req.r.p.pdiag_show = PACKET_SHOW_INFO | PACKET_SHOW_MCLIST | PACKET_SHOW_FANOUT | PACKET_SHOW_RING_CFG;
 	tmp = do_collect_req(nl, &req, sizeof(req), packet_receive_one, collect_err, ns, &req.r.p);
 	if (tmp)
 		err = tmp;
 
-	req.r.n.sdiag_family	= AF_NETLINK;
-	req.r.n.sdiag_protocol	= NDIAG_PROTO_ALL;
-	req.r.n.ndiag_show	= NDIAG_SHOW_GROUPS;
+	req.r.n.sdiag_family = AF_NETLINK;
+	req.r.n.sdiag_protocol = NDIAG_PROTO_ALL;
+	req.r.n.ndiag_show = NDIAG_SHOW_GROUPS;
 	tmp = do_collect_req(nl, &req, sizeof(req), netlink_receive_one, collect_err, ns, &req.r.n);
 	if (tmp)
 		err = tmp;
